import functools
from typing import Any, Callable, Dict

from hyperopt import fmin, tpe, STATUS_OK
from hyperopt.spark import SparkTrials


class HyperoptTuner:
    """
    A class for hyperparameter tuning using hyperopt.

    Attributes:
        search_space: A dictionary representing the search space.
        max_evals: The maximum number of evaluations.
        spark_trials: An instance of SparkTrials to distribute the tuning across a Spark cluster.

    Methods:
        __call__: A method that makes the class instance callable. It wraps the passed function and
        optimizes the function's parameters according to the search space.
    """

    def __init__(
        self,
        search_space: Dict[str, Any],
        max_evals: int,
        spark_trials: SparkTrials = None,
    ) -> None:
        """
        The constructor for the HyperoptTuning class.
        """
        self.search_space = search_space
        self.max_evals = max_evals
        self.spark_trials = spark_trials

    def __call__(self, func: Callable[..., Any]) -> Callable[..., Any]:
        """
        Makes the class instance callable.
        """

        @functools.wraps(func)
        def wrapped_func(**kwargs: Any) -> Dict[str, Any]:
            """
            The wrapped function that will run the optimization process.
            """

            def objective(params: Dict[str, Any]) -> Dict[str, Any]:
                """
                The objective function to be minimized.
                """
                return {
                    "loss": func(params, **kwargs),
                    "status": STATUS_OK,
                }

            best_params = fmin(
                objective,
                self.search_space,
                algo=tpe.suggest,
                max_evals=self.max_evals,
                trials=self.spark_trials,
            )
            return best_params

        return wrapped_func


def using_hyperopt_tuning(
    search_space: Dict[str, Any], max_evals: int, spark_trials: SparkTrials = None
) -> Callable:
    """
    A convenience function for using the HyperoptTuner class as a decorator.

    Args:
        search_space (Dict[str, Any]): A dictionary representing the search space.
        max_evals (int): The maximum number of evaluations.
        spark_trials (SparkTrials, optional): An instance of SparkTrials to distribute the tuning across a Spark cluster
        Defaults to None.

    Returns:
        Callable: A decorator for tuning hyperparameters using hyperopt.
    """
    tuner = HyperoptTuner(search_space, max_evals, spark_trials)

    def decorator(func: Callable[..., Any]) -> Callable[..., Any]:
        return tuner(func)

    return decorator



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 2.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import curia.api.curia.api.curia.api.curia.api.swagger_client
from curia.api.curia.api.curia.api.curia.api.swagger_client.api.cohorts_api import CohortsApi  # noqa: E501
from curia.api.curia.api.curia.api.curia.api.swagger_client.rest import ApiException


class TestCohortsApi(unittest.TestCase):
    """CohortsApi unit test stubs"""

    def setUp(self):
        self.api = CohortsApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_cohort_controller_query(self):
        """Test case for cohort_controller_query

        Submit Cohort Queries  # noqa: E501
        """
        pass

    def test_cohort_controller_sql(self):
        """Test case for cohort_controller_sql

        Generate SQL queries for cohort  # noqa: E501
        """
        pass

    def test_create_many_base_cohort_controller_cohort(self):
        """Test case for create_many_base_cohort_controller_cohort

        Create many Cohort  # noqa: E501
        """
        pass

    def test_create_one_base_cohort_controller_cohort(self):
        """Test case for create_one_base_cohort_controller_cohort

        Create one Cohort  # noqa: E501
        """
        pass

    def test_delete_one_base_cohort_controller_cohort(self):
        """Test case for delete_one_base_cohort_controller_cohort

        Delete one Cohort  # noqa: E501
        """
        pass

    def test_get_many_base_cohort_controller_cohort(self):
        """Test case for get_many_base_cohort_controller_cohort

        Retrieve many Cohort  # noqa: E501
        """
        pass

    def test_get_one_base_cohort_controller_cohort(self):
        """Test case for get_one_base_cohort_controller_cohort

        Retrieve one Cohort  # noqa: E501
        """
        pass

    def test_replace_one_base_cohort_controller_cohort(self):
        """Test case for replace_one_base_cohort_controller_cohort

        Replace one Cohort  # noqa: E501
        """
        pass

    def test_update_one_base_cohort_controller_cohort(self):
        """Test case for update_one_base_cohort_controller_cohort

        Update one Cohort  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



from typing import Any


class hybridmethod:
    """
    A decorator that allows for the creation of a method that can be called as a classmethod or an instancemethod.
    See https://stackoverflow.com/questions/28237955/same-name-for-classmethod-and-instancemethod
    """
    def __init__(self, fclass: type, finstance: object = None, doc: str = None):
        self.fclass = fclass
        self.finstance = finstance
        self.__doc__ = doc or fclass.__doc__
        # support use on abstract base classes
        self.__isabstractmethod__ = bool(
            getattr(fclass, '__isabstractmethod__', False)
        )

    def classmethod(self, fclass: type):
        return type(self)(fclass, self.finstance, None)

    def instancemethod(self, finstance: object):
        return type(self)(self.fclass, finstance, self.__doc__)

    def __get__(self, instance: object, cls: type) -> Any:
        if instance is None or self.finstance is None:
            # either bound to the class, or no instance method available
            return self.fclass.__get__(cls, None)
        return self.finstance.__get__(instance, cls)
